#!/usr/bin/php
<?php
// *********************************************
// *                                           *
// * Apache Check by Franck Hisbergue          *
// * https://xulops.net/forge/apachecheck.php  *
// *                                           *
// * Usage (with user root) :                  *
// *     php apachecheck_vx.x.x.php            *
// *                                           *
// * Use --help to get all the options         *
// *                                           *
// * This script is under Apache 2.0 License   *
// *                                           * 
// *********************************************

$script_version = "1.5.1";
$apache_port    = 80; // default port apache is listening to (used to find the pid)

// *******************************************
// *******************************************
// *             SOME FUNCTIONS              *
// *******************************************
// *******************************************

function display($text) {
	// display text on 76 cols max
	$first_line = false;
	$t = explode("\r\n", $text);
	foreach ($t as $text) {
		if (! $first_line) {
			if (strlen($text) > 76) {
				for ($i = 76; $i > 0; $i--) {
					if ($text[$i] == " ") {
						echo substr($text, 0, $i)."\r\n"; // first line
						$text = trim(substr($text, $i));
						break;
					}
				}
				if ($i == 0) {
					echo substr($text, 0, 76)."\r\n"; // first line
					$text = substr($text, 76);
				}
			} else {
				echo $text."\r\n";
				$text = "";
			}
			$first_line = true;
		}
		if ($text != "") {
			// other lines, with 5 spaces -> 71 cars
			do {
				if (strlen($text) > 71) {
					for ($i = 71; $i > 0; $i--) {
						if ($text[$i] == " ") {
							echo "     ".substr($text, 0, $i)."\r\n";
							$text = trim(substr($text, $i));
							break;
						}
					}
					if ($i == 0) {
						echo "     ".substr($text, 0, 71)."\r\n";
						$text = substr($text, 71);
					}
				} else {
					echo "     ".$text."\r\n";
					$text = "";
				}
			} while (strlen($text) > 0);
		}
	}
}

function display_title($title) {
	$title = "-------- ".$title." ";
	$title = "\r\n".$title.str_repeat("-", 76 - strlen($title))."\r\n";
	echo $title;
}

function human_filesize($bytes, $decimals = 1) {
	$bytes = round($bytes, 0);
	$sz = ' KMGTP';
	$factor = floor((strlen($bytes) - 1) / 3);
	return trim(sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$sz[$factor]).'o';
}

function human_duration($duration) {
	// convert secondes to human readable duration
	$days = floor($duration / (24 * 3600));
	$rest = $duration - ($days * 24 * 3600);
	$hours = floor($rest / 3600);
	$rest = $rest - ($hours * 3600);
	$minutes = floor($rest / 60);
	$secondes = floor($rest - ($minutes * 60));
	$ret = "";
	if ($days > 0) $ret .= "$days day".($days > 1?"s ":" ");
	if ($hours > 0) $ret .= "$hours hour".($hours > 1?"s ":" ");
	if ($minutes > 0) $ret .= "$minutes minute".($minutes > 1?"s ":" ");
	if ($secondes > 0) $ret .= "$secondes seconde".($secondes > 1?"s":"");
	return trim($ret);
}

function get_param_from_line($line, $n, $sep = " ") {
	// return the nth parameter of the line (spaces or tabs between parameters)
	$ret = "";
	if ($sep == " ") {
		$t = explode(" ", str_replace("\t", " ", $line));
	} else {
		$t = explode($sep, $line);
	}
	$tt = array();
	foreach ($t as $l) {
		if (trim($l) != "") array_push($tt, trim($l));
	}
	if (isset($tt[$n])) {
		$ret = $tt[$n];
	}
	return $ret;
}

function get_mem_usage($pid) {
	// return an array with private and shared memory in octets of a process
	$privat = 0;
	$privat_huge = 0;
	$pss = 0;
	$shared = 0;
	if (file_exists("/proc/$pid/smaps_rollup")) {
		$smaps = file_get_contents("/proc/$pid/smaps_rollup"); // it is faster to use this file, but not present on old systems
	} else {
		$smaps = file_get_contents("/proc/$pid/smaps"); // it is slowest to use this file, but more compatible
	}
	$t = explode("\n", $smaps);
	foreach ($t as $line) {
		if ((strpos($line, "Private_Clean:") !== false) || (strpos($line, "Private_Dirty:") !== false)) {
			$l1 = get_param_from_line($line, 1);
			if (is_numeric($l1)) {
				$privat += $l1;
			}
		}
		if (stripos($line, "pss:") !== false) {
			$l1 = get_param_from_line($line, 1);
			if (is_numeric($l1)) {
				$pss += $l1;
			}
		}
		if (stripos($line, "private_hugetlb:") !== false) {
			$l1 = get_param_from_line($line, 1);
			if (is_numeric($l1)) {
				$privat_huge += $l1;
			}
		}
	}
	$shared = $pss - $privat;
	$privat += $privat_huge;
	return array($privat * 1024, $shared * 1024);
}

function get_all_children_pid($pid) {
	// recursive function to get all the children (and children of children...) pid of a parent pid
	global $pid_childs;
	if ($h1 = @opendir("/proc")) {
		while (false !== ($file1 = readdir($h1))) {
			if (is_numeric($file1)) {
				$stat = @file_get_contents("/proc/$file1/stat");
				$l3 = get_param_from_line($stat, 3);
				if ($l3 != "") {
					if ($l3 == $pid) {
						$l0 = get_param_from_line($stat, 0);
						array_push($pid_childs, $l0);
						get_all_children_pid($l0);
					}
				}
			}
		}
		closedir($h1);
	}
}

function get_pid_io($pid) {
	// get pid io disk usage by using /proc/{pid}/io   kernel 2.6.20 minimum
	// rd = read from disk only
	// wd = write to disk only
	// rn = number of reads
	// wn = number of writes
	// st = process life in seconds
	global $kernel_version, $tck, $system_uptime;
	$rd = 0;
	$wd = 0;
	$rn = 0;
	$wn = 0;
	$st = 0;
	if ($kernel_version > 20620) {
		$io = file_get_contents("/proc/$pid/io");
		$t = explode("\n", $io);
		foreach ($t as $line) {
			if (substr($line, 0, 11) == "read_bytes:") $rd = (int) get_param_from_line($line, 1);
			if (substr($line, 0, 12) == "write_bytes:") $wd = (int) get_param_from_line($line, 1);
			if (substr($line, 0, 6) == "syscw:") $wn = (int) get_param_from_line($line, 1);
			if (substr($line, 0, 6) == "syscr:") $rn = (int) get_param_from_line($line, 1);
		}
	}
	// get starttime of this process
	$start_time = 0;
	$stat = file_get_contents("/proc/$pid/stat");
	$start_time = get_param_from_line($stat, 21);
	$start_time /= $tck;
	$st = $system_uptime - $start_time;
	return array($rd, $wd, $rn, $wn, $st);
}

function linearize_version($v) {
	// this function gets version in format x.y.z and return it in x0y0z format for comparisons
	$major = get_param_from_line($v, 0, ".");
	$minor = str_pad(get_param_from_line($v, 1, "."), 2, "0", STR_PAD_LEFT);
	$third = str_pad(get_param_from_line($v, 2, "."), 2, "0", STR_PAD_LEFT);
	return $major.$minor.$third;
}

function get_pid_cpu_usage($pid) {
	// this funtion gets /proc/[PID]/stat informations to calculate this process CPU usage
	// utime     - CPU time spent in user code, measured in clock ticks
	// stime     - CPU time spent in kernel code, measured in clock ticks
	// cutime    - Waited-for children's CPU time spent in user code (in clock ticks)
	// cstime    - Waited-for children's CPU time spent in kernel code (in clock ticks)
	// starttime - Time when the process started, measured in clock ticks
	global $tck;
	// systeme uptime
	$system_uptime = trim(str_replace("\t", " ", file_get_contents("/proc/uptime")));
	$system_uptime = substr($system_uptime, 0, strpos($system_uptime, " "));
	// stat infos
	$stat = file_get_contents("/proc/$pid/stat");
	$utime      = (int) get_param_from_line($stat, 13);
	$stime      = (int) get_param_from_line($stat, 14);
	$cutime     = (int) get_param_from_line($stat, 15);
	$cstime     = (int) get_param_from_line($stat, 16);
	$start_time = (int) get_param_from_line($stat, 21);
	$total_time = ($utime + $stime + $cutime + $cstime) / $tck;
	$start_time /= $tck;
	$pid_uptime = $system_uptime - $start_time;
	$cpu_usage = 100 * $total_time / $pid_uptime;
	return $cpu_usage;
}

function apache_cmd($param) {
	// call apachectl, apache2ctl, httpd, ... with the $cmd parameter and return result
	global $process_cmdline;
	switch ($process_cmdline) {
		case "/usr/sbin/httpd" : 
			$cmd = "$process_cmdline $param 2>&1";
			break;
		case "/usr/sbin/httpd.worker" :
			$cmd = "$process_cmdline $param 2>&1";
			break;
		case "/usr/sbin/apache2" :
			$cmd = "/usr/sbin/apache2ctl $param 2>&1";
			break;
		case "/usr/local/apache/bin/httpd" :
			if ($param == "-V") display("[@@] Seems apache has been installed from source");
			$cmd = "$process_cmdline $param 2>&1";
			break;
		case "/opt/apache2/bin/httpd" :
			if ($param == "-V") display("[@@] Seems apache has been installed from built package");
			$cmd = "$process_cmdline $param 2>&1";
			break;
		default :
			$cmd = "";
		// fin switch
	}
	if ($cmd == "") {
		display("[!!] Fatal error retreiving Apache version, please check your configuration");
		exit();
	}
	return $cmd;
}

// *******************************************
// *******************************************
// *          START of MAIN SCRIPT           *
// *******************************************
// *******************************************

// *******************************************
// *         COMMAND LINE OPTIONS            *
// *******************************************

$options = "hidcmalp:";
$opts = getopt($options, array("help", "no-internet", "skip-io", "skip-cpu", "skip-memory", "skip-log", "apache-config", "port:"));
// Handle command line arguments
for ($i = 0; $i < strlen($options); $i++) ${"arg_".$options[$i]} = false;
foreach (array_keys($opts) as $opt) {
	switch ($opt) {
		case "h":
		case "help":
			$arg_h = true; break;
		case "i":
		case "no-internet" :
			$arg_i = true; break;
		case "d":
		case "skip-io" :
			$arg_d = true; break;
		case "c":
		case "skip-cpu" :
			$arg_c = true; break;
		case "m":
		case "skip-memory" :
			$arg_m = true; break;
		case "l":
		case "skip-log" :
			$arg_l = true; break;
		case "a":
		case "apache-config" :
			$arg_a = true; break;
		case "p":
		case "port":
			$apache_port = (int) $opts[$opt];
		// fin switch
	}
}

// Hostname
$hostname = "";
$cmd_hostname = trim(shell_exec("which hostname"));
if ($cmd_hostname != "") {
	$hostname = trim(shell_exec("$cmd_hostname -f"));
} else {
	display("[!!] hostname command not found");
}

// external ip
if ($arg_i) {
	$ip = "x.x.x.x";
} else {
	if (! $ip = @file_get_contents("https://xulops.net/bin/ip.php")) $ip = "x.x.x.x";
}

// get last version number of this script
if ($arg_i) {
	$last_script_version = $script_version;
} else {
	$last_script_version = trim(@file_get_contents("https://xulops.net/forge/apachecheck.php?f=lastversion"));
	if ($last_script_version == "") $last_script_version = $script_version;
}

// print header
display(" ");
display(">> Apache Check v$script_version - Hisbergue Franck - for $hostname ($ip)");
if (linearize_version($last_script_version) > linearize_version($script_version)) {
	display(">> A new version is available : $last_script_version");
}
display(">> For bug reports, feature requests, comments or FAQ : https://xulops.net/forge/apachecheck.php".($arg_h?"":"\r\nUse --help to get all the options"));
display(" ");
display(">> DISCLAIMER : this script is provided 'as is', without warranty of any kind. You use the software at your own risk, and you are solely responsible for the consequences of modifications you may apply to your system.");
display(">> Licensed under the Apache License, Version 2.0. You may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0\r\nUnless required by applicable law or agreed to in writing, software distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.");

// help on command line
if ($arg_h) {
	display(" ");
	display(">> Glossary :\r\n[..] : information\r\n[@@] : warning\r\n[>>] : recommandation\r\n[!!] : critical problem");
	display(" ");
	display(">> usage (using root or sudo) : php apachecheck_v$script_version.php [option]\r\nOptions :");
	display("    -a       --apache-config   Display Apache config files content");
	display("    -c       --skip-cpu        Don't do CPU usage measures and checks");
	display("    -d       --skip-io         Don't do disk IO usage measures and checks");
	display("    -h       --help            Display this help and exit");
	display("    -i       --no-internet     Don't get IP and don't check for new version");
	display("    -l       --skip-log        Don't do large log checks");
	display("    -m       --skip-memory     Don't do memoru usage measures and checks");
	display("    -p port  --port            Change port (default 80)");
	display(" ");
	exit();
}
display(" ");

// test port
if ($apache_port < 1 && $apache_port > 65535) {
	display("[!!] Fatal error : port must be between 1 and 65535");
	exit();
}

// root ?
$is_root = trim(shell_exec("id -u"));
if ($is_root != 0) {
	display("[!!] Fatal error : this script must run as root user");
	exit();
}

// *******************************************
// *              SERVER INFOS               *
// *******************************************

display_title("Server informations");
// OS name, version
$os_type = php_uname("s");
$os_version = php_uname("v");
$os_release = php_uname("r");
if (strtolower($os_type) != "linux") {
	display("[!!] Fatal error : OS is not a linux ($os_type)\r\n");
	exit();
} else {
	display("[..] OS type : $os_type");
	display("[..] OS version : $os_version");
	display("[..] OS release : $os_release");
	$kernel_version = linearize_version(substr($os_release, 0, strpos($os_release, "-")));
}

// system uptime
$system_uptime = trim(str_replace("\t", " ", file_get_contents("/proc/uptime")));
$system_uptime = substr($system_uptime, 0, strpos($system_uptime, " "));
display("[..] System uptime : ".human_duration($system_uptime)."");

// tick per seconds for uptime calculations
$tck = trim(shell_exec("getconf CLK_TCK"));
if (! is_numeric($tck)) {
	display("[@@] CLK_TCK is not a number");
	$tck = 100;
}

// Server CPU infos
if ($arg_c) {
	display("[..] CPU detection skipped");
} else {
	$cpuinfo = trim(file_get_contents("/proc/cpuinfo"));
	$t = explode("\n\n", str_replace("\r\n", "\n", str_replace("\t", " ", $cpuinfo)));
	$t_cpu  = array();
	$t_core = array();
	$t_ht   = array();
	foreach ($t as $cpuinfo_bloc) {
		$tt = explode("\n", $cpuinfo_bloc);
		$processor   = "";
		$core_id     = "";
		$physical_id = "";
		foreach ($tt as $line) {
			if (strpos($line, "processor") !== false) $processor = get_param_from_line($line, 1, ":");
			if (strpos($line, "core id") !== false) $core_id = get_param_from_line($line, 1, ":");
			if (strpos($line, "physical id") !== false) $physical_id = get_param_from_line($line, 1, ":");
		}
		if (! in_array($physical_id, $t_cpu)) array_push($t_cpu, $physical_id);
		if (! in_array($physical_id."-".$core_id, $t_core)) array_push($t_core, $physical_id."-".$core_id);
		if (! in_array($physical_id."-".$core_id."-".$processor, $t_ht)) array_push($t_ht, $physical_id."-".$core_id."-".$processor);
	}
	$nb_cpu  = count($t_cpu);
	$nb_core = count($t_core);
	$nb_ht   = count($t_ht); // hyperthreading get avg 30% better cpu
	$cpu_server_limit = ($nb_core + ($nb_ht - $nb_core) * 30 / 100) * 100;  
	display("[..] $nb_cpu CPU, $nb_core cores, $nb_ht hyperthreaded");
}

// server total mem and available memory
$server_total_mem = 0;
$server_mem_available = 0;
if ($arg_m) {
	display("[..] Server total memory : skipped");
} else {
	$meminfo = trim(file_get_contents("/proc/meminfo"));
	$t = explode("\n", str_replace("\t", " ", $meminfo));
	foreach ($t as $line) {
		if (stripos($line, "MemTotal:") !== false) {
			$server_total_mem = (int) get_param_from_line($line, 1);
			$server_total_mem *= 1024;
		}
		if (stripos($line, "MemAvailable:") !== false) {
			$server_mem_available = (int) get_param_from_line($line, 1);
			$server_mem_available *= 1024;
		}
	}
	if ($server_total_mem == 0) {
		display("[!!] Fatal error : unable to determine server total memory");
		exit();
	} else {
		if ($server_mem_available < 0) {
			display("[!!] Server total memory : ".human_filesize($server_total_mem).", No available memory : ".human_filesize($server_mem_available));
		} else {
			display("[..] Server total memory : ".human_filesize($server_total_mem).", available memory : ".human_filesize($server_mem_available));
		}
	}
}

// check total disk size and available size
$disk_total_size = disk_total_space(".");
$disk_free_size = disk_free_space(".");
if ($disk_free_size / $disk_total_size * 100 < 5) {
	display("[!!] Less than 5% of disk free space left : ".human_filesize($disk_free_size)." free on a total of ".human_filesize($disk_total_size));
} else {
	display("[..] Disk free space : ".human_filesize($disk_free_size)." on a total of ".human_filesize($disk_total_size));
} 

// mesure disk IO performance
$nb_pow_2 = 12; // file between 256o AND 512Ko
$will_write = 256 * pow(2, $nb_pow_2 + 1);
$disk_io_speed = 0;
if ($arg_d) {
	display("[..] Disk IO performance skipped");
} else {
	if ($disk_free_size < $will_write) {
		display("[!!] Disk IO performance skipped : need ".human_filesize($will_write)." free on disk");
	} else {
		// test if 'sync' and 'dd' are available
		$which_sync = trim(shell_exec("which sync"));
		$which_dd = trim(shell_exec("which dd"));
		if ($which_sync == "") {
			display("[!!] Command 'sync' not found, disk IO performance skipped");
		} else {
			if ($which_dd == "") {
				display("[!!] Command 'dd' not found, disk IO performance skipped");
			} else {
				// flush caches
				$ret = shell_exec("sync");
				$ret = shell_exec("echo 3 > /proc/sys/vm/drop_caches");

				// generate a 256 cars string with all possible 256 cars in random order
				$str_256_1 = "";
				$nb_bytes_rw = 256;
				for ($i = 0; $i < 256; $i++) $str_256_1 .= chr($i);
				$str_256_1 = str_shuffle($str_256_1);
				
				// creation of str pow of 2 in random order
				for ($i = 2; $i <= $nb_pow_2; $i++) {
					${"str_256_".$i} = str_shuffle(str_repeat($str_256_1, pow(2, $i - 1)));
					$nb_bytes_rw += strlen(${"str_256_".$i});
				}
				
				// writing strings on disk
				for ($i = 1; $i <= $nb_pow_2; $i++) {
					file_put_contents("./apachecheck_file_str_256_".$i, ${"str_256_".$i});
				}
				
				// writing speed
				$ret = shell_exec("sync");
				$nb_byte_w = 0;
				$time_w = 0;
				for ($i = 1; $i <= $nb_pow_2; $i++) {
					$deb = microtime(true);
					$ret = shell_exec("dd if=./apachecheck_file_str_256_$i of=apachecheck_file_str_256_2_$i oflag=direct 2>&1");
					$fin = microtime(true);
					@unlink("./apachecheck_file_str_256_$i");
					@unlink("./apachecheck_file_str_256_2_$i");
					$nb_byte_w += 256 * pow(2, $i - 1);
					$time_w += $fin - $deb;
				}
				$disk_io_speed = $nb_byte_w / $time_w;
				display("[..] Estimated residual disk IO speed : ".human_filesize($disk_io_speed)."/s");
			}
		}
	}
}

// *******************************************
// *          Apache basic infos             *
// *******************************************
display_title("Apache basic informations");

// check apachectl
$apachectl = trim(shell_exec("which apachectl"));
if ($apachectl != "") {
	display("[OK] apachectl exists ($apachectl)");
} else {
	// no, try to find apache2tl
	$apachectl = trim(shell_exec("which apache2ctl"));
	if ($apachectl != "") {
		display("[OK] apache2ctl exists ($apachectl)");
	} else {
		display("[!!] Fatal error : apachectl or apache2ctl not found");
		exit();
	}
}

// pid of apache main proccess
$hex_port = str_pad(dechex($apache_port), 4, "0", STR_PAD_LEFT);
$tcp = file_get_contents("/proc/net/tcp")."\n".file_get_contents("/proc/net/tcp6");
$t = explode("\n", $tcp);
$inode = "";
foreach ($t as $line) {
	$l9 = get_param_from_line($line, 9);
	if ($l9 != 0) {
		$l1 = get_param_from_line($line, 1);
		$l1_1 = get_param_from_line($l1, 1, ":");
		if ($l1_1 == $hex_port) {
			$inode = $l9;
		}
	}
}
if ($inode == "" || $inode == 0) {
	display("[!!] Fatal error : inode not found on /proc/net/tcp(6) for port $apache_port (hex : $hex_port)");
	exit();
}
// search for the pid with this inode in /proc/<pid>/fd  (need to be root)
$apache_pid = "";
$process_name = "";
$process_cmdline = "";
if ($h1 = @opendir("/proc")) {
	$found = false;
    while (false !== ($file1 = readdir($h1))) {
		if (! $found) {
			if (is_numeric($file1)) {
				if ($h2 = @opendir("/proc/$file1/fd")) {
					while (false !== ($file2 = readdir($h2))) {
						if (! $found) {
							if (is_link("/proc/$file1/fd/$file2")) {
								$file3 = readlink("/proc/$file1/fd/$file2");
								if (strpos($file3, "socket:[$inode]") !== false) {
									display("[..] Found inode for port $apache_port in $file3 in /proc/$file1/fd");
									$status = file_get_contents("/proc/$file1/status");
									$status_ppid = "";
									$status_pid = "";
									$status_lines = explode("\n", $status);
									foreach ($status_lines as $line) {
										if (substr($line, 0, 4) == "Pid:") {
											$status_pid = trim(substr($line, 4));
										}
										if (substr($line, 0, 5) == "PPid:") {
											$status_ppid = trim(substr($line, 5));
										}
										if (substr($line, 0, 5) == "Name:") {
											$process_name = trim(substr($line, 5));
										}
									}
									if ($status_ppid == 1) {
										$apache_pid = $status_pid;
									} else {
										$apache_pid = $status_ppid;
									}
									$process_cmdline = readlink("/proc/$file1/exe");
									$found = true;
								}
							}
						}
					}
					closedir($h2);
				}
			}
		}
    }
    closedir($h1);
}
if ($apache_pid == "") {
	display("[!!] Fatal error : unable to find the main apache process pid in /proc for inode $inode");
	exit();
} else {
	display("[OK] Apache main process '$process_name' found : $process_cmdline (pid $apache_pid)");
}

// get Apache build parameters : version, root directory, config file
$build_param = shell_exec(apache_cmd("-V"));
$t = explode("\n", $build_param);
$apache_version     = "";
$mod_mpm_temp       = "";
$apache_root        = "";
$apache_config_file = "";
foreach ($t as $line) {
	if (strpos($line, "Server version") !== false) {
		$apache_version = get_param_from_line($line, 1, ":");
	}
	if (strpos($line, "Server MPM") !== false) {
		$mod_mpm_temp = get_param_from_line($line, 1, ":");
	}
	if (strpos($line, "HTTPD_ROOT") !== false) {
		$apache_root = trim(str_replace('"', "", substr($line, strpos($line, "=") + 1)));
	}
	if (strpos($line, "SERVER_CONFIG_FILE") !== false) {
		$apache_config_file = trim(str_replace('"', "", substr($line, strpos($line, "=") + 1)));
	}
	if (strpos($line, "DEFAULT_PIDLOG") !== false) {
		$apache_default_pidlog = trim(str_replace('"', "", substr($line, strpos($line, "=") + 1)));
	}
}
if ($apache_version != "") {
	display("[..] Apache version : $apache_version");
} else {
	display("[!!] Error : unable to find Apache version");
	exit();
}
$apache_version = substr($apache_version, strpos($apache_version, "/") + 1);
$apache_version = substr($apache_version, 0, strpos($apache_version, " "));
$apache_version = linearize_version($apache_version);
if (! is_dir($apache_root)) {
	display("[!!] Fatal error : unable to detect apache root directory");
	exit();
} else {
	display("[..] Apache root directory : $apache_root");
}
if ($apache_config_file[0] != "/") $apache_config_file = $apache_root."/".$apache_config_file;
if (! is_file($apache_config_file)) {
	display("[!!] Fatal error : unable to detect apache config file");
	exit();
} else {
	display("[..] Main apache config file : $apache_config_file");
}

// check config file syntax
$configtest = array();
$apache_config_syntax = exec(apache_cmd("configtest"), $configtest, $result_configtest);
if ($result_configtest > 0) {
	display("[!!] Apache config files syntax is incorrect, see the syntax check result below :\r\n".implode("\r\n", $configtest));
	exit();
} else {
	display("[OK] Apache config files syntax is correct");
}

// get MPM model and other modules loaded
$mod_prefork = 0;
$mod_worker = 0;
$mod_event = 0;
$mod_itk = false;
$mod_http2 = false;
$mod_fcgi = false;
$mod_log_config = false;
$modules = shell_exec(apache_cmd("-M"));
$t = explode("\n", $modules);
foreach ($t as $line) {
	if (strpos($line, "mpm_prefork_module") !== false) $mod_prefork = 1;
	if (strpos($line, "mpm_worker_module") !== false) $mod_worker = 1;
	if (strpos($line, "mpm_event_module") !== false) $mod_event = 1;
	if (strpos($line, "mpm_itk_module") !== false) $mod_itk = true;
	if (strpos($line, "http2_module") !== false) $mod_http2 = true;
	if (strpos($line, "proxy_fcgi_module") !== false) $mod_fcgi = true;
	if (strpos($line, "log_config_module") !== false) $mod_log_config = true;
}
if ($mod_prefork + $mod_worker + $mod_event == 0) {
	display("[!!] Fatal error : unable to get MPM module");
	exit();
}
if ($mod_prefork + $mod_worker + $mod_event > 1) {
	$t_mod_mpm = array();
	if ($mod_prefork == 1) array_push($t_mod_mpm, "prefork");
	if ($mod_worker == 1) array_push($t_mod_mpm, "worker");
	if ($mod_event == 1) array_push($t_mod_mpm, "event");
	display("[!!] Fatal error : Only one MPM module can be loaded (".implode(" + ", $t_mod_mpm).")");
	exit();
}
$mod_mpm = "";
if ($mod_mpm_temp == "prefork" && $mod_prefork == 1) $mod_mpm = "prefork";
if ($mod_mpm_temp == "worker" && $mod_worker == 1) $mod_mpm = "worker";
if ($mod_mpm_temp == "event" && $mod_event == 1) $mod_mpm = "event";
if ($mod_mpm == "") {
	display("[!!] Fatal error : MPM module loaded doesn't match Apache command line MPM server ($mod_mpm_temp)");
	exit();
}
display("[..] MPM used : $mod_mpm".($mod_mpm == "prefork"?" (itk: ".($mod_itk?"installed":"not-installed").")":"")."");

// apache uptime
$start_time = 0;
$stat = file_get_contents("/proc/$apache_pid/stat");
$start_time = get_param_from_line($stat, 21);
if ($start_time == "") {
	display("[!!] Error : unable to get starting time of the parent process in /proc/$apache_pid/stat");
} else {
	$start_time /= $tck;
}
$apache_uptime = $system_uptime - $start_time;
if ($start_time != 0) {
	display("[..] Apache uptime : ".human_duration($apache_uptime)."");
}
if ($apache_uptime < 3600) {
	display("[!!] Apache started for less than one hour, reports may be inaccurate");
}

// *******************************************
// *      Get apache config parameters       *
// *******************************************

// Read entire config files (with all includes)
$cont = true;
$conf_all = file_get_contents($apache_config_file);
while ($cont) {
	$t = explode("\n", $conf_all);
	$i = 0;
	$cont = false;
	$conf_all2 = "";
	while ($i < count($t)) {
		if (trim($t[$i]) != "") {
			if (substr(trim($t[$i]), 0, 1) != "#") {  // we don't care comments
				if (substr(rtrim($t[$i]), -1) == "\\") {
					$conf_all2 .= substr(rtrim($t[$i]), 0, -1); // line with backslash at the end, no CR/NL
					$cont = true;
				} else {
					if (stripos($t[$i], "include") !== false) {
						// get the include condition
						$l0 = get_param_from_line($t[$i], 0);
						$l1 = get_param_from_line($t[$i], 1);
						if (stripos($l0, "include") !== false) {
							if ($l1 != "") {
								if ($l1[0] == "/") {
									$t_inc = glob($l1);
								} else {
									$t_inc = glob($apache_root."/".$l1);
								}
								foreach ($t_inc as $inc) {
									$conf_all2 .= file_get_contents($inc)."\r\n";
								}
								$cont = true;
							}
						}
					} else {
						$conf_all2 .= $t[$i]."\r\n";
					}
				}
			}
		}
		$i++;
	}
	$conf_all = trim($conf_all2);
}

// read envvars if exists
if (file_exists("$apache_root/envvars")) {
	$envvars = file_get_contents("$apache_root/envvars");
	$t = explode("\n", $envvars);
	foreach ($t as $line) {
		if (trim($line) != "") {
			if (substr(trim($line), 0, 1) != "#") {  // we don't care comments
				$l0 = get_param_from_line($line, 0);
				if (strtolower($l0) == "export") {
					$l1 = get_param_from_line($line, 1);
					$l1_0 = get_param_from_line($l1, 0, "=");
					$l1_1 = trim(str_replace('$SUFFIX', "", get_param_from_line($l1, 1, "=")));
					$conf_all = str_replace('${'.$l1_0.'}', $l1_1, $conf_all);
				}
			}
		}
	}
}

// apache params to read from config files
// 1: param is unique, if multiples values encountred, only keep the last one
// c: param is unique but has severals values in the same line, all the values are kept
// m: param isn't unique, values are stored in an array
// n: c+m, param isn't unique and have several values on the same line 
$apache_param = array(
	"User"                     => "1:apache_user",
	"PidFile"                  => "1:apache_pidfile",
	"HostnameLookups"          => "1:hostname_lookups",
	"ServerSignature"          => "1:server_signature",
	"ErrorLog"                 => "m:apache_error_log",
	"TimeOut"                  => "1:time_out",
	"KeepAlive"                => "1:keep_alive",
	"KeepAliveTimeout"         => "1:keep_alive_timeout",
	"MaxKeepAliveRequests"     => "1:max_keep_alive_requests",
	"ServerLimit"              => "1:server_limit",
	"MaxRequestsPerChild"      => "1:max_request_per_child",
	"MaxConnectionsPerChild"   => "1:max_connections_per_child",
	"MaxClients"               => "1:max_clients",
	"MaxRequestWorkers"        => "1:max_request_workers",
	"StartServers"             => "1:start_servers",
	"MinSpareServers"          => "1:min_spare_servers",
	"MaxSpareServers"          => "1:max_spare_servers",
	"ThreadsPerChild"          => "1:threads_per_child",
	"ThreadLimit"              => "1:thread_limit",
	"MinSpareThreads"          => "1:min_spare_threads",
	"MaxSpareThreads"          => "1:max_spare_threads",
	"AsyncRequestWorkerFactor" => "1:async_request_worker_factor",
	"Protocols"                => "n:protocols",
	"BufferedLogs"             => "1:buffered_logs",
	"ServerName"               => "m:server_name",
	"ServerAlias"              => "n:server_alias"
	
	);
foreach ($apache_param as $param => $varparam) {
	// init des variables
	$varname = substr($varparam, 2);
	${$varname} = array(); 
}
$t = explode("\n", $conf_all);
foreach ($t as $line) {
	$l0 = get_param_from_line($line, 0);
	$l1 = get_param_from_line($line, 1);
	foreach ($apache_param as $param => $varparam) {
		$varname = substr($varparam, 2);
		if (strtolower($l0) == strtolower($param)) {
			if ($l1 != "") {
				if ($varparam[0] == "1" || $varparam[0] == "m") {
					array_push(${$varname}, $l1);
				} else {
					if ($varparam[0] == "c" || $varparam[0] == "n") {
						// all the param of this line, after $l0
						$val = trim(substr(trim($line), strlen($l0)));
						array_push(${$varname}, $val);
					}
				}
			}
		}
	}
}
// check single or multiple values
foreach ($apache_param as $param => $varparam) {
	$varname = substr($varparam, 2);
	if ($varparam[0] == "1" || $varparam[0] == "c") {
		// must be unique, only care the last one
		if (count(${$varname}) == 0) {
			${$varname} = "";
		} else {
			${$varname} = ${$varname}[count(${$varname}) - 1];
		}
	}
}
	
// *******************************************
// *   Apache common config file parameters  *
// *******************************************
display_title("Apache common config file parameters");

// find apache user
if ($apache_user == "") {
	display("[@@] unable to find apache user in conf file");
} else {
	// apache user id
	$apache_user_id = trim(shell_exec("id -u $apache_user"));
	if ($apache_user != "apache" && $apache_user != "www-data") {
		display("[@@] apache user is not-standard : $apache_user (id $apache_user_id)");
	} else {
		display("[..] apache user : $apache_user (id $apache_user_id)");
	}
}

// pid file
$apache_pidfile = str_replace('"', "", $apache_pidfile);
if ($apache_pidfile == "") $apache_pidfile = $apache_default_pidlog;
if ($apache_pidfile == "") {
	display("[@@] unable to find apache PidFile in conf file");
} else {
	$apache_pidfile_pid = trim(file_get_contents($apache_pidfile));
	if ($apache_pid != $apache_pidfile_pid) {
		display("[@@] Apache Pid in $apache_pidfile doesn't match Pid listening port $apache_port");
	} else {
		display("[OK] Apache Pid in $apache_pidfile matches port listener pid");
	}
}

if ($time_out > 300) {
	display("[>>] Unless specific usacase, timeOut is too high, should be 300 (actually $time_out)");
} else {
	if ($time_out < 30) {
		display("[>>] TimeOut is too low, should be at least 30, typically 300 (actually $time_out)");
	} else {
		display("[OK] TimeOut value is correct : $time_out");
	}
}

if (strtolower($hostname_lookups) != "off") {
	display("[>>] HostnameLookups should be Off, unless you really need it (actually $hostname_lookups)");
} else {
	display("[OK] HostnameLookups is Off");
}

if (strtolower($server_signature) == "on") {
	display("[>>] ServerSignature should be Off, unless you really need it (actually $server_signature)");
} else {
	display("[OK] ServerSignature is Off");
}

if (strtolower($keep_alive) == "on") {
	display("[..] KeepAlive is 'On'");
	if ($max_keep_alive_requests < 100 || $max_keep_alive_requests > 500) {
		display("[>>] MaxKeepAliveRequests should be between 100 and 500 for better performance (actually $max_keep_alive_requests)");
	} else {
		display("[OK] MaxKeepAliveRequests is between 100 and 500 : $max_keep_alive_requests");
	}
	if ($keep_alive_timeout > 15) {
		display("[>>] Unless specific usacase, KeepAliveTimeout should be less than 15, and typically 5 (actually $keep_alive_timeout)");
	} else {
		display("[OK] KeepAliveTimeout is less than 15 : $keep_alive_timeout");		
	}
} else {
	display("[>>] KeepAlive should be 'On' to serve more quickly several requests from a visitor");
}

if ($mod_log_config && strtolower($buffered_logs) != "on") {
	display("[>>] BufferedLogs is Off, put it 'On' may reduce disk writes");
}

// get virtualhosts informations
$virtual_hosts = shell_exec(apache_cmd("-S"));
$t = explode("\n", $virtual_hosts);
$t_vhosts = array();
$t_vhosts_name = array();
foreach ($t as $line) {
	if (substr(trim($line), 0, 4) == "port") {
		$port = (int) get_param_from_line($line, 1);
		if ($port > 0) {
			if (! isset($t_vhosts[$port])) $t_vhosts[$port] = 0;
			$t_vhosts[$port]++;
			$name = get_param_from_line($line, 3);
			if ($name != "") array_push($t_vhosts_name, $name);
		}
	}
}
$nb_virtualhost = array_sum($t_vhosts);
$list_vhosts_per_port = "";
foreach ($t_vhosts as $port => $nb) {
	if ($list_vhosts_per_port != "") $list_vhosts_per_port .= ", ";
	$list_vhosts_per_port .= "$nb on port $port";
}
display("[..] $nb_virtualhost VirtualHost found : $list_vhosts_per_port");

// check if vhotst names are in /etc/hosts
// add all ServerName and ServerAlias to the list to check
foreach ($server_name as $name) {
	$p = parse_url($name);
	if (isset($p["host"])) {
		if (! in_array($p["host"], $t_vhosts_name)) array_push($t_vhosts_name, $p["host"]);
	}
}
foreach ($server_alias as $names) {
	$i = 0;
	do {
		$name = get_param_from_line($names, $i);
		if ($name != "") {
			if (! in_array($name, $t_vhosts_name)) array_push($t_vhosts_name, $name);
		}
		$i++;
	} while ($name != "");
}
$hosts = @file_get_contents("/etc/hosts");
$t = explode("\n", $hosts);
$t_hosts = array();
foreach ($t as $line) {
	$ip = get_param_from_line($line, 0);
	if ($ip != "#") {
		$i = 1;
		do {
			$name = get_param_from_line($line, $i);
			if ($name != "") array_push($t_hosts, $name);
			$i++;
		} while ($name != "");
	}
}
$vhosts_not_found_in_hosts_file = array();
foreach ($t_vhosts_name as $vname) {
	if (! in_array($vname, $t_hosts)) {
		array_push($vhosts_not_found_in_hosts_file, $vname);
	}
}
if (count($vhosts_not_found_in_hosts_file) > 0) {
	display("[@@] VirtualHost name".(count($vhosts_not_found_in_hosts_file) > 1?"s":"")." not found in /etc/hosts : ".implode(", ", $vhosts_not_found_in_hosts_file));
} else {
	display("[OK] All virtualhost names are in /etc/hosts");
}

if ($mod_mpm == "prefork") {
	// *******************************************
	// *      prefork MPM config parameters      *
	// *******************************************
	display_title("prefork MPM config parameters");
	
	// ServerLimit
	if ($server_limit == "") {
		display("[@@] unable to find apache ServerLimit value in conf file, will use default value (256)");
		$server_limit = 256;
	} else {
		display("[..] ServerLimit found : $server_limit");
	}
	
	// MaxConnectionsPerChild / MaxRequestsPerChild
	if ($max_connections_per_child == "") $max_connections_per_child = $max_requests_per_child;
	if ($max_connections_per_child == "") {
		$max_connections_per_child = 0;
		display("[@@] MaxConnectionsPerChild / MaxRequestsPerChild not found, assuming 0 (infinite)");
	}
	if ($max_connections_per_child == 0) {
		display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild shouldn't be zero to avoid long term memory leaks (usually high value around 10000)");
	} else {
		if ($max_connections_per_child < 1000) {
			display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild ($max_connections_per_child) should be >= 1000 to avoid frequent process creation (usually around 10000)");
		} else {
			if ($max_connections_per_child > 1000000) {
				display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild ($max_connections_per_child) shouldn't be too high to avoid long term memory leaks (usually around 10000)");
			} else {
				display("[OK] MaxConnectionsPerChild / MaxRequestsPerChild is correct : $max_connections_per_child");
			}
		}
	}

	// MaxClients and MaxRequestWorkers
	if ($max_request_workers == "") $max_request_workers = $max_clients;
	if ($max_request_workers == "") {
		display("[@@] unable to find apache MaxRequestWorkers / MaxClients value in conf file, will use default value (256)");
		$max_request_workers = 256;
	} else {
		display("[..] MaxRequestWorkers / MaxClients found : $max_request_workers");
	}
	if ($max_request_workers > $server_limit) {
		display("[@@] MaxRequestWorkers / MaxClients ($max_request_workers) is higher than ServerLimit ($server_limit), using ServerLimit");
		$max_request_workers = $server_limit;
	}

	// MinSpareServers and MaxSpareServers
	if ($min_spare_servers == "") {
		$min_spare_servers = 5;
		display("[@@] MinSpareServers not found, using default value (5)");
	}
	if ($max_spare_servers == "") {
		$max_spare_servers = 10;
		display("[@@] MaxSpareServers not found, using default value (10)");
	}
	if ($min_spare_servers > 5 * $server_total_mem / 1073741824) { // max 5 MinSpareServers per Go of physical memory
		display("[>>] Regarding server physical memory MinSpareServers ($min_spare_servers) should be equal or less than ".floor(5 * $server_total_mem / 1073741824)."");
	} else {
		display("[OK] MinSpareServers value is correct : $min_spare_servers");
	}
	if ($max_spare_servers <= $min_spare_servers) {
		display("[!!] MaxSpareServers ($max_spare_servers) must be higher than MinSpareServers ($min_spare_servers)");
	} else {
		if ($max_spare_servers > 10 * $min_spare_servers) {
			display("[>>] MaxSpareServers ($max_spare_servers) should be 2 to 5 times MinSpareServers ($min_spare_servers)");
		} else {
			display("[OK] MaxSpareServers value is correct : $max_spare_servers");
		}
	}
		
	// StartServers
	if ($start_servers == "") $start_servers = 5;
	if ($start_servers < $min_spare_servers) {
		display("[>>] StartServers ($start_servers) should be equal or higher than MinSpareServers ($min_spare_servers)");
	} else {
		if ($start_servers > $max_spare_servers) {
			display("[>>] StartServers ($start_servers) should be equal or less than MaxSpareServers ($max_spare_servers)");
		} else {
			display("[OK] StartServers value is correct : $start_servers");
		}
	}

	// number of virtualhosts vs MaxRequestWorkers
	if ($nb_virtualhost > $max_request_workers) {
		display("[@@] The number of VirtualHost is higher than MaxRequestWorkers / MaxClients, this is unusual");
	}
}

if ($mod_mpm == "worker" || $mod_mpm == "event") {
	// *******************************************
	// * worker and event MPM config parameters  *
	// *******************************************
	display_title("$mod_mpm MPM config parameters");
	
	// ServerLimit
	if ($server_limit == "") {
		display("[@@] unable to find apache ServerLimit value in conf file, will use default value for worker and event MPM (16)");
		$server_limit = 16;
	} else {
		display("[..] ServerLimit found : $server_limit");
	}
	
	// MaxConnectionsPerChild / MaxRequestsPerChild
	if ($max_connections_per_child == "") $max_connections_per_child = $max_requests_per_child;
	if ($max_connections_per_child == "") {
		$max_connections_per_child = 0;
		display("[@@] MaxConnectionsPerChild / MaxRequestsPerChild not found, assuming 0 (infinite)");
	}
	if ($max_connections_per_child == 0) {
		display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild shouldn't be zero to avoid long term memory leaks (usually high value around 10000)");
	} else {
		if ($max_connections_per_child < 1000) {
			display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild ($max_connections_per_child) should be >= 1000 to avoid frequent process creation (usually around 10000)");
		} else {
			if ($max_connections_per_child > 1000000) {
				display("[>>] MaxConnectionsPerChild / MaxRequestsPerChild ($max_connections_per_child) shouldn't be too high to avoid long term memory leaks (usually around 10000)");
			} else {
				display("[OK] MaxConnectionsPerChild / MaxRequestsPerChild is correct : $max_connections_per_child");
			}
		}
	}
	
	// http/2 activated
	$h2 = false;
	$h2c = false;
	if ($mod_http2) {
		if (count($protocols) > 0) {
			foreach ($protocols as $line) {
				$t = explode(" ", str_replace("\t", " ", $line));
				foreach ($t as $val) {
					if (trim($val) == "h2") $h2 = true;
					if (trim($val) == "h2c") $h2c = true;
				}
			}
			
		}
		if ($h2 || $h2c) {
			$http2_disp = "[..] HTTP/2 is activated : ";
			if ($h2 && $h2c) {
				$http2_disp .= "HTTP/2 over TLS and HTTP/2 over TCP";
			} else {
				if ($h2) $http2_disp .= "HTTP/2 over TLS";
				if ($h2c) $http2_disp .= "HTTP/2 over TCP";
			}
			display($http2_disp);
		} else {
			display("[..] HTTP/2 is not activated");
		}
	}
	
	// ThreadsPerChild and ThreadLimit
	if ($threads_per_child == "") {
		display("[@@] unable to find apache ThreadsPerChild value in conf file, will use default value for worker and event MPM (25)");
		$threads_per_child = 25;
	} else {
		display("[..] ThreadsPerChild found : $threads_per_child");
	}
	if ($thread_limit == "") {
		display("[@@] unable to find apache ThreadLimit value in conf file, will use default value for worker and event MPM (64)");
		$thread_limit = 64;
	} else {
		display("[..] ThreadLimit found : $thread_limit");
	}
	if ($threads_per_child > $thread_limit) {
		display("[!!] ThreadsPerChild ($threads_per_child) cannot be higher than ThreadLimit ($thread_limit)");
	} else {
		if ($threads_per_child < $thread_limit) {
			display("[>>] ThreadLimit ($thread_limit) is higher than ThreadsPerChild ($threads_per_child), wasting memory a little, ThreadLimit can be adjust to ThreadsPerChild");
		} else {
			if ($threads_per_child < 20) {
				display("[>>] ThreadsPerChild ($threads_per_child) is low, threads sharing is not optimal (usually 64)");
			} else {
				if ($threads_per_child > 20000) {
					display("[@@] ThreadsPerChild ($threads_per_child) is high, you should decrease it and increase MaxRequestWorkers accordingly");
				} else {
					display("[OK] ThreadLimit value is correct : $thread_limit");
				}
			}
		}
	}	
	
	// MinSpareThreads and MaxSpareThreads
	if ($min_spare_threads == "") {
		$min_spare_threads = 75;
		display("[@@] MinSpareThreads not found, using default value (75)");
	} else {
		display("[..] MinSpareThreads found : $min_spare_threads");
	}
	if ($max_spare_threads == "") {
		$max_spare_threads = 250;
		display("[@@] MaxSpareThreads not found, using default value (250)");
	} else {
		display("[..] MaxSpareThreads found : $max_spare_threads");
	}
	if ($max_spare_threads <= $min_spare_threads) {
		display("[>>] MaxSpareThreads ($max_spare_threads) should be higher than MinSpareThreads ($min_spare_threads)");
	}
	if ($min_spare_threads > 50 * $server_total_mem / 1073741824) { // max 50 MinSpareThreads per Go of physical memory
		display("[>>] Regarding server physical memory MinSpareThreads ($min_spare_threads) should be equal or less than ".floor(50 * $server_total_mem / 1073741824)."");
	} else {
		if ($min_spare_threads < 25 && $server_total_mem > 1073741824) { // min 25 MinSpareThreads
			display("[>>] MinSpareThreads ($min_spare_threads) should be at least 25");
		} else {
			if (($h2 || $h2c) && $min_spare_threads < 50 && $server_total_mem > 1073741824) {
				display("[>>] With HTTP/2 activated, it is better MinSpareThreads >= 50 to avoid threads creations on HTTP/2 multiplexing requests.");
			} else {
				display("[OK] MinSpareThreads value is correct : $min_spare_servers");
			}
		}
	}
	if ($max_spare_threads <= $min_spare_threads) {
		display("[!!] MaxSpareThreads ($max_spare_threads) must be higher than MinSpareThreads ($min_spare_threads)");
	} else {
		if ($max_spare_threads > 10 * $min_spare_threads) {
			display("[>>] MaxSpareThreads ($max_spare_threads) should be 2 to 5 times MinSpareThreads ($min_spare_threads)");
		} else {
			display("[OK] MaxSpareThreads value is correct : $max_spare_threads");
		}
	}
	
	// StartServers
	if ($start_servers == "") {
		display("[@@] StartServers not found, using default value (3)");
		$start_servers = 3;
	} else {
		display("[..] StartServers found : $start_servers");
	}
	if ($max_spare_threads < $start_servers * $threads_per_child) {
		display("[>>] MaxSpareThreads ($max_spare_threads) shoud be higher than StartServers * ThreadsPerChild ($start_servers * $threads_per_child)");
	}
	if ($min_spare_threads < $threads_per_child) {
		display("[>>] MinSpareThreads ($min_spare_threads) should be at least equal to ThreadsPerChild ($threads_per_child)");
	}
	
	// MaxClients and MaxRequestWorkers
	if ($max_request_workers == "") $max_request_workers = $max_clients;
	if ($max_request_workers == "") {
		display("[@@] unable to find apache MaxRequestWorkers / MaxClients value in conf file, will use default value (16)");
		$max_request_workers = 16;
	} else {
		display("[..] MaxRequestWorkers / MaxClients found : $max_request_workers");
	}
	if ($max_request_workers > $threads_per_child * $server_limit) {
		display("[!!] MaxRequestWorkers ($max_request_workers) is higher than ServerLimit * ThreadsPerChild ($server_limit * $threads_per_child), use ServerLimit * ThreadsPerChild instead");
		$max_request_workers = $server_limit * $threads_per_child;
	} else {
		if ($max_request_workers < $threads_per_child * $server_limit) {
			display("[@@] ServerLimit * ThreadsPerChild ($server_limit * $threads_per_child = ".($server_limit * $threads_per_child).") is higher than MaxRequestWorkers ($max_request_workers)");
		} else {
			display("[OK] MaxRequestWorkers value is correct : $max_request_workers");
		}
	}
	
	// AsyncRequestWorkerFactor
	if ($async_request_worker_factor == "") {
		display("[@@] AsyncRequestWorkerFactor not found, using default value (2)");
		$async_request_worker_factor = 2;
	}
	if ($async_request_worker_factor != 2) {
		display("[@@] AsyncRequestWorkerFactor ($async_request_worker_factor) has an unusual value (2)");
	}
	
	// number of virtualhosts
	if ($nb_virtualhost > $max_request_workers) {
		display("[@@] The number of VirtualHost is higher than MaxRequestWorkers / MaxClients, this is unusual");
	}
}

// *******************************************
// *             Large log check             *
// *******************************************
if ($arg_l) {
	display_title("Large log check skipped");
} else {
	display_title("Large log check");
	// check large logs
	$log_files = array(
		"/var/log/httpd",
		"/var/log/httpd24",
		"/var/log/apache2",
		"/var/log/php-fpm",
		"/usr/local/apache/logs",
		"/usr/local/apache2/logs",
		"/usr/local/httpd/logs"
		);
	$large_log_files = 0;
	foreach ($log_files as $log_file) {
		if (is_dir($log_file)) {
			$dir_log_files = glob($log_file."/*");
			foreach ($dir_log_files as $dir_log_file) {
				if (file_exists($dir_log_file)) {
					if (filesize($dir_log_file) > 1024 * 1024 *1024) {
						$large_log_files++;
						display("[!!] Log file $dir_log_file is more than 1Go large, this is abnormal, consider using rotate log");
					}
				}
			}
		} else {
			if (file_exists($log_file)) {
				if (filesize($log_file) > 1024 * 1024 *1024) {
					$large_log_files++;
					display("[!!] Log file $log_file is more than 1Go large, this is abnormal, consider using rotate log");
				}
			}
		}
	}
	if ($large_log_files == 0) {
		display("[OK] No large log files");
	}

	// check if Maxclients has been reached
	$maxclients_reached = 0;
	$error_log_files = array(
		"/var/log/httpd/error_log",
		"/var/log/httpd24/error_log",
		"/usr/local/apache/logs/error_log"
		);
	// add error_log files from conf_all
	foreach ($apache_error_log as $apache_log_file) {
		if (! in_array($apache_log_file, $error_log_files)) array_push($error_log_files, $apache_log_file);
	}
	// check these log files
	foreach ($error_log_files as $error_log_file) {
		if (file_exists($error_log_file)) {
			$f = fopen($error_log_file, "r");
			while (! feof($f)) {
				$line = fgets($f);
				if (stripos($line, "reached")) {
					$maxclients_reached++;
				}
			}
			fclose($f);
		}
	}
	if ($maxclients_reached > 0) {
		display("[!!] Recently, MaxClients / MaxRequestWorkers has been reached $maxclients_reached time".($maxclients_reached?"s":"")."");
	} else {
		display("[OK] MaxClients / MaxRequestWorkers hasn't been reached recently");
	}
}

// *******************************************
// *      check PHP-FPM (worker / event)     *
// *******************************************

$phpfpm_cpu = 0;
$phpfpm_total_mem_usage = 0;
$phpfpm_child_max_mem_usage = 0;
$phpfpm_child_avg_mem_usage = 0;
$phpfpm_nb_childs = 0;
$phpfpm_io_rd = 0;
$phpfpm_io_wd = 0;
$phpfpm_io_rs = 0;
$phpfpm_io_ws = 0;

if ($mod_mpm == "worker" || $mod_mpm == "event") {
	if ($mod_fcgi) {
		display_title("PHP-FPM check");
		// get php-pfm parent process id
		$phpfpm_pid = "";
		if ($h1 = @opendir("/proc")) {
			$found = false;
			while (false !== ($file1 = readdir($h1))) {
				if (! $found) {
					if (is_numeric($file1)) {
						$status = file_get_contents("/proc/$file1/status");
						$status_lines = explode("\n", $status);
						$pid_id = "";
						$parent_pid = "";
						$name = "";
						foreach ($status_lines as $line) {
							if (substr($line, 0, 4) == "Pid:") {
								$pid_id = trim(substr($line, 4));
							}
							if (substr($line, 0, 5) == "PPid:") {
								$parent_pid = trim(substr($line, 5));
							}
							if (substr($line, 0, 5) == "Name:") {
								$name = trim(substr($line, 5));
							}
						}
						if (strpos($name, "php-fpm") !== false && $parent_pid == "1") {
							$found = true;
							$phpfpm_pid = $pid_id;
						}
					}
				}
			}
			closedir($h1);
		}
		if ($phpfpm_pid == "") {
			display("[..] PHP-FPM process not found");
		} else {
			display("[..] PHP-FPM process found : $phpfpm_pid");
			// get php-fpm childs process id
			$phpfpm_child_pid = array();
			$pid_childs = array();
			get_all_children_pid($phpfpm_pid);
			$phpfpm_child_pid = $pid_childs;
			$phpfpm_nb_childs = count($phpfpm_child_pid);
			if ($phpfpm_nb_childs == 0) {
				display("[@@] PHP-FPM childs processes not found");
			} else {
				display("[..] PHP-FPM childs processes found : $phpfpm_nb_childs");
			}
			// mem usage of parent phpfpm pid
			list($privat, $shared) = get_mem_usage($phpfpm_pid);
			$phpfpm_parent_mem_usage = $privat + $shared; // we count all for the main process
			if ($phpfpm_parent_mem_usage > 52428800) {
				display("[!!] PHP-FPM parent process memory usage over 50Mo (".human_filesize($phpfpm_parent_mem_usage).")");
			} else {
				display("[OK] PHP-FPM parent process memory usage : ".human_filesize($phpfpm_parent_mem_usage)."");
			}
			
			// PHP-FPM CPU usage
			if (! $arg_c) {
				$phpfpm_cpu = get_pid_cpu_usage($phpfpm_pid);
				foreach ($phpfpm_child_pid as $child_pid) {
					$phpfpm_cpu += get_pid_cpu_usage($child_pid);
				}
				if ($phpfpm_cpu > $cpu_server_limit) {
					display("[!!] PHP-FPM CPU usage (".round($phpfpm_cpu, 2)."%) is too high");
				} else {
					display("[..] PHP-FPM CPU usage : ".round($phpfpm_cpu, 2)."%");
				}
			}
			
			// php-fpm total memory usage
			if (! $arg_m) {
				$phpfpm_total_mem_usage = $phpfpm_parent_mem_usage;
				$phpfpm_child_min_mem_usage = "999999999999999";
				$phpfpm_child_max_mem_usage = 0;
				$phpfpm_child_avg_mem_usage = 0;
				$phpfpm_nb_childs = 0;
				$phpfpm_shared_cum = 0;
				// get the memory usage of all the childs of php-fpm
				foreach ($phpfpm_child_pid as $child_pid) {
					list($privat, $shared) = get_mem_usage($child_pid);
					$phpfpm_child_mem_usage = $privat;
					$phpfpm_shared_cum += $shared;
					$phpfpm_total_mem_usage += $phpfpm_child_mem_usage;
					$phpfpm_child_avg_mem_usage += $phpfpm_child_mem_usage;
					$phpfpm_nb_childs++;
					if ($phpfpm_child_mem_usage < $phpfpm_child_min_mem_usage) $phpfpm_child_min_mem_usage = $phpfpm_child_mem_usage;
					if ($phpfpm_child_mem_usage > $phpfpm_child_max_mem_usage) $phpfpm_child_max_mem_usage = $phpfpm_child_mem_usage;
				}
				// avg shared usage repartition
				$phpfpm_shared_avg = ceil($phpfpm_shared_cum / $phpfpm_nb_childs / $phpfpm_nb_childs);
				$phpfpm_child_min_mem_usage += $phpfpm_shared_avg;
				$phpfpm_child_max_mem_usage += $phpfpm_shared_avg;
				$phpfpm_total_mem_usage += $phpfpm_shared_avg * $phpfpm_nb_childs;
				// echo memory usage
				display("[..] Total PHP-FPM memory usage : ".human_filesize($phpfpm_total_mem_usage)."");
				display("[..] Smallest PHP-FPM child memory usage : ".human_filesize($phpfpm_child_min_mem_usage)."");
				display("[..] Biggest PHP-FPM child memory usage : ".human_filesize($phpfpm_child_max_mem_usage)."");
				if ($phpfpm_nb_childs > 0) {
					$phpfpm_child_avg_mem_usage = ceil ($phpfpm_child_avg_mem_usage / $phpfpm_nb_childs) + $phpfpm_shared_avg;
					display("[..] Average PHP-FPM child memory usage : ".human_filesize($phpfpm_child_avg_mem_usage)."");
				} else {
					display("[!!] No PHP-FPM child process found during memory usage check");
					exit();
				}
			}
			
			// IO disk usage parent process
			if (! $arg_d) {
				$phpfpm_io = array();
				list($rd, $wd, $rn, $wn, $st) = get_pid_io($phpfpm_pid);
				$phpfpm_io[$phpfpm_pid]["rd"] = $rd;
				$phpfpm_io[$phpfpm_pid]["wd"] = $wd;
				$phpfpm_io[$phpfpm_pid]["st"] = $st;
				// IO disk usage
				foreach ($phpfpm_child_pid as $child_pid) {
					list($rd, $wd, $rn, $wn, $st) = get_pid_io($child_pid);
					$phpfpm_io[$child_pid]["rd"] = $rd;
					$phpfpm_io[$child_pid]["wd"] = $wd;
					$phpfpm_io[$child_pid]["st"] = $st;
				}
				// should remove pm.min_spare_servers number of childs... maybe later
				// phpfpm average io disk usage
				$phpfpm_avg_rd = 0;
				$phpfpm_avg_wd = 0;
				$phpfpm_avg_st = 0;
				foreach ($phpfpm_io as $child_pid => $io) {
					$phpfpm_avg_rd += $io["rd"];
					$phpfpm_avg_wd += $io["wd"];
					$phpfpm_avg_st += $io["st"];
				}
				$phpfpm_io_rd = $phpfpm_avg_rd / $phpfpm_avg_st;
				$phpfpm_io_wd = $phpfpm_avg_wd / $phpfpm_avg_st;
				display ("[..] PHP-FPM disk IO : ".human_filesize($phpfpm_io_rd, 0)."/s read, ".human_filesize($phpfpm_io_wd, 0)."/s write");
			}
		}
	}
}

// *******************************************
// *       Check  Mysql / MariaDB            *
// *******************************************
display_title("MySql check");

$mysql_cpu = 0;
$mysql_mem_usage_per_threads = 0;
$mysql_nb_childs = 0;
$mysql_io_rd = 0;
$mysql_io_wd = 0;

// get mysql user id
$mysql_uid = (int) trim(shell_exec("id -u mysql 2> /dev/null"));
if ($mysql_uid < 2) {
	display("[@@] unable to find mysql user");
} else {
	display("[..] mysql user found, id $mysql_uid");
	// get mysql parent process id
	$mysql_pid = "";
	if ($h1 = @opendir("/proc")) {
		$found = false;
		while (false !== ($file1 = readdir($h1))) {
			if (! $found) {
				if (is_numeric($file1)) {
					$status = file_get_contents("/proc/$file1/status");
					$status_lines = explode("\n", $status);
					$pid_id = "";
					$parent_pid = "";
					$user_id = "";
					foreach ($status_lines as $line) {
						if (substr($line, 0, 4) == "Pid:") {
							$pid_id = trim(substr($line, 4));
						}
						if (substr($line, 0, 5) == "PPid:") {
							$parent_pid = trim(substr($line, 5));
						}
						if (substr($line, 0, 4) == "Uid:") {
							$user_id = trim(substr($line, 4));
						}
					}
					if ($user_id != "") {
						$user_id = get_param_from_line($user_id, 0);
						if ($user_id == $mysql_uid && $parent_pid == "1") {
							$found = true;
							$mysql_pid = $pid_id;
						}
					}
				}
			}
		}
		closedir($h1);
	}
	if ($mysql_pid == "") {
		display("[..] MySql parent process not found");
	} else {
		display("[..] MySql process id found : $mysql_pid");
		// get number of mysql threads
		$mysql_child_pid = array();
		if ($h1 = @opendir("/proc/$mysql_pid/task")) {
			while (false !== ($file1 = readdir($h1))) {
				if (is_numeric($file1)) {
					if ($file1 != $mysql_pid) array_push($mysql_child_pid, $file1);
				}
			}
			closedir($h1);
		}
		$mysql_nb_childs = count($mysql_child_pid);
		if ($mysql_nb_childs == 0) {
			display("[@@] MySql threads not found");
		} else {
			display("[..] MySql threads found : $mysql_nb_childs");
		}
		
		// MySql CPU usage
		if (! $arg_c) {
			$mysql_cpu = get_pid_cpu_usage($mysql_pid);
			if ($mysql_cpu > $cpu_server_limit) {
				display("[!!] MySql CPU usage (".round($mysql_cpu, 2)."%) is too high");
			} else {
				display("[..] MySql CPU usage : ".round($mysql_cpu, 2)."%");
			}
		}
			
		// mem usage of parent mysql pid
		if (! $arg_m) {
			list($privat, $shared) = get_mem_usage($mysql_pid);
			$mysql_parent_mem_usage = $privat + $shared; // we count all for the main process
			display("[..] MySql total memory usage : ".human_filesize($mysql_parent_mem_usage)."");
			// memory per thread
			$mysql_mem_usage_per_threads = 2936000;  // about 2.8Mo (yes, it is approximative, use mysqltuner to get the real value)
		}
		
		// MySql io disk usage
		if (! $arg_d) {
			list($rd, $wd, $rn, $wn, $st) = get_pid_io($mysql_pid);
			$mysql_io_rd = $rd / $st;
			$mysql_io_wd = $wd / $st;
			display("[..] MySql disk IO : ".human_filesize($mysql_io_rd, 0)." read, ".human_filesize($mysql_io_wd, 0)." write");
		}
	}
}

// *******************************************
// *         Apache child processes          *
// *******************************************
display_title("Apache child processes");

// check id of child processes of parent process
$child_uid = "";
if ($h1 = @opendir("/proc")) {
	$found = false;
    while (false !== ($file1 = readdir($h1))) {
		if (! $found) {
			if (is_numeric($file1)) {
				if ($file1 != $apache_pid) {
					$status = file_get_contents("/proc/$file1/status");
					$status_lines = explode("\n", $status);
					$user_id = "";
					$parent_pid = "";
					foreach ($status_lines as $line) {
						if (substr($line, 0, 4) == "Uid:") {
							$user_id = trim(substr($line, 4));
						}
						if (substr($line, 0, 5) == "PPid:") {
							$parent_pid = trim(substr($line, 5));
						}
					}
					if ($parent_pid == $apache_pid) {
						$found = true;
						$child_uid = $user_id;
					}
	
				}
			}
		}
    }
    closedir($h1);
}
if (trim($child_uid) == "") {
	display("[@@] unable to find the uid of child processes");
} else {
	$child_uid = get_param_from_line($child_uid, 0);
	// get the user name
	$child_u_name = "";
	$passwd = file_get_contents("/etc/passwd");
	$t = explode("\n", $passwd);
	foreach ($t as $line) {
		$l2 = get_param_from_line($line, 2, ":");
		if ($l2 != "") {
			if ($l2 == $child_uid) {
				$l0 = get_param_from_line($line, 0, ":");
				$child_u_name = $l0;
			}
		}
	}
	display("[..] Detected child processes uid : $child_uid ($child_u_name)");
}
if ($apache_user_id != $child_uid) {
	if (! $mod_itk) {
		display("[@@] Child processes uid doesn't match config file user");
	}
}

// number and id of child processes (call a recursive function to get all children of children ...)
$apache_child_pid = array();
$pid_childs = array();
get_all_children_pid($apache_pid);
$apache_child_pid = $pid_childs;
$apache_nb_childs = count($apache_child_pid);
if ($apache_nb_childs == 0) {
	display("[!!] Error : Apache childs processes not found");
	exit();
} else {
	display("[..] Apache childs processes found : $apache_nb_childs");
}
if ($apache_nb_childs > $max_request_workers) {
	display("[@@] Apache child processes found is higher than MaxRequestWorkers / MaxClients");
}

// remove the apache preforked children (prefork = MinSpareServers, worker & event = MinSpareThreads / ThreadsPerChild)
// based on rn = number of reads
$remove_nb_childs = 0;
if ($mod_mpm == "prefork") {
	$remove_nb_childs = $min_spare_servers;
}
if ($mod_mpm == "worker" || $mod_mpm == "event") {
	$remove_nb_childs = ceil($min_spare_threads / $threads_per_child);
}

// *******************************************
// *             Apache CPU usage            *
// *******************************************
$apache_cpu = 0;
if (! $arg_c) {
	$apache_cpus = array();
	array_push($apache_cpus, get_pid_cpu_usage($apache_pid));
	foreach ($apache_child_pid as $child_pid) {
		array_push($apache_cpus, get_pid_cpu_usage($child_pid));
	}
	sort($apache_cpus, SORT_NUMERIC);
	for ($i = 0; $i < $remove_nb_childs; $i++) {
		unset($apache_cpus[$i]);
	}
	$apache_cpu = array_sum($apache_cpus);
	if ($apache_cpu > $cpu_server_limit) {
		display("[!!] Apache CPU usage (".round($apache_cpu, 2)."%) is too high");
	} else {
		display("[..] Apache CPU usage : ".round($apache_cpu, 2)."%");
	}
}

// *******************************************
// *              apache IO disk             *
// *******************************************
$apache_io_rd = 0;
$apache_io_wd = 0;
if (! $arg_d) {
	// parent process
	$t_io_rd = array();
	$t_io_wd = array();
	$t_io_st = array();
	list($rd, $wd, $rn, $wn, $st) = get_pid_io($apache_pid);
	$t_io_rd[$apache_pid] = $rd;
	$t_io_wd[$apache_pid] = $wd;
	$t_io_st[$apache_pid] = $st;
	foreach ($apache_child_pid as $child_pid) {
		list($rd, $wd, $rn, $wn, $st) = get_pid_io($child_pid);
		$t_io_rd[$child_pid] = $rd;
		$t_io_wd[$child_pid] = $wd;
		$t_io_st[$child_pid] = $st;
	}
	asort($t_io_rd);
	$i = 0;
	foreach ($t_io_rd as $pid => $val) {
		if ($i < $remove_nb_childs) {
			unset($t_io_rd[$pid]);
			unset($t_io_wd[$pid]);
			unset($t_io_st[$pid]);
		}
		$i++;
	}
	// apache children average io disk usage
	$ac_avg_rd = 0;
	$ac_avg_wd = 0;
	$ac_avg_st = 0;
	foreach ($t_io_rd as $pid => $rd) {
		$wd = $t_io_rd[$pid];
		$st = $t_io_st[$pid];
		$ac_avg_rd += $rd;
		$ac_avg_wd += $wd;
		$ac_avg_st += $st;
	}
	$apache_io_rd = $ac_avg_rd / $ac_avg_st;
	$apache_io_wd = $ac_avg_wd / $ac_avg_st;
	display ("[..] Apache disk IO : ".human_filesize($apache_io_rd, 0)."/s read, ".human_filesize($apache_io_wd, 0)."/s write");
}

// *******************************************
// *          Apache Memory usage            *
// *******************************************

$child_avg_mem_usage = 0;
$child_max_mem_usage = 0;
if (! $arg_m) {
	// mem usage of parent pid
	list($privat, $shared) = get_mem_usage($apache_pid);
	$apache_parent_mem_usage = $privat + $shared; // we count all for the main process
	if ($apache_parent_mem_usage > 52428800) {
		display("[!!] Parent process memory usage over 50Mo (".human_filesize($apache_parent_mem_usage).")");
	} else {
		display("[OK] Parent process memory usage : ".human_filesize($apache_parent_mem_usage)."");
	}

	// Apache total memory usage
	$apache_total_mem_usage = $apache_parent_mem_usage;
	$child_min_mem_usage = 999999999999999;
	$child_max_mem_usage = 0;
	$child_avg_mem_usage = 0;
	$apache_nb_childs = 0;
	$shared_cum = 0;
	// get the memory usage of all the children
	$t_privat = array();
	$t_shared = array();
	foreach ($apache_child_pid as $child_pid) {
		list($privat, $shared) = get_mem_usage($child_pid);
		$t_privat[$child_pid] = $privat;
		$t_shared[$child_pid] = $shared;
	}
	asort($t_privat);
	$i = 0;
	foreach ($t_privat as $pid => $val) {
		if ($i < $remove_nb_childs) {
			unset($t_privat[$pid]);
			unset($t_shared[$pid]);
		}
		$i++;
	}

	foreach ($t_privat as $pid => $privat) {
		$shared = $t_shared[$pid];
		$child_mem_usage = $privat;
		$shared_cum += $shared;
		$apache_total_mem_usage += $child_mem_usage;
		$child_avg_mem_usage += $child_mem_usage;
		$apache_nb_childs++;
		if ($child_mem_usage < $child_min_mem_usage) $child_min_mem_usage = $child_mem_usage;
		if ($child_mem_usage > $child_max_mem_usage) $child_max_mem_usage = $child_mem_usage;
	}
	// avg shared usage repartition
	$shared_avg = ceil($shared_cum / $apache_nb_childs / $apache_nb_childs);
	$child_min_mem_usage += $shared_avg;
	$child_max_mem_usage += $shared_avg;
	$apache_total_mem_usage += $shared_avg * $apache_nb_childs;
	// echo memory usage
	display("[..] Total apache memory usage : ".human_filesize($apache_total_mem_usage)."");
	display("[..] Smallest child memory usage : ".human_filesize($child_min_mem_usage)."");
	display("[..] Biggest child memory usage : ".human_filesize($child_max_mem_usage)."");
	if ($apache_nb_childs > 0) {
		$child_avg_mem_usage = ceil ($child_avg_mem_usage / $apache_nb_childs) + $shared_avg;
		display("[..] Average child memory usage : ".human_filesize($child_avg_mem_usage)."");
	} else {
		display("[!!] No child process found during memory usage check");
		exit();
	}
}

// ********************************************************
// *          Memory, CPU and IO considerations           *
// ********************************************************
if (! $arg_m || ! $arg_c || ! $arg_d) {
	// title
	$title = array("Memory", "CPU", "IO");
	if ($arg_d) unset($title[2]);
	if ($arg_c) unset($title[1]);
	if ($arg_m) unset($title[0]);
	switch (count($title)) {	
		case 3 : display_title("Memory, CPU and IO considerations (all included)"); break;
		case 2 : display_title(implode(" and ", $title)." considerations (all included)"); break;
		case 1 :display_title(implode(" ", $title)." considerations (all included)"); break;
	}

	// avg memory usage estimated for one apache child, including PHP-Fpm and MySql
	$actual_max_workers = $max_request_workers;
	if ($mod_mpm == "worker" || $mod_mpm == "event") $actual_max_workers = floor($max_request_workers / $threads_per_child);
	if (! $arg_m) {
		$avg_per_child = $child_avg_mem_usage;
		if ($mysql_nb_childs > 0) $avg_per_child += $mysql_mem_usage_per_threads * $mysql_nb_childs / $apache_nb_childs;
		if ($phpfpm_nb_childs > 0) $avg_per_child += $phpfpm_child_avg_mem_usage * $phpfpm_nb_childs / $apache_nb_childs;
		// max memory usage estimated for one apache child, including PHP-Fpm and MySql
		$max_per_child = $child_max_mem_usage;
		if ($mysql_nb_childs > 0) $max_per_child += $mysql_mem_usage_per_threads * $mysql_nb_childs / $apache_nb_childs;
		if ($phpfpm_nb_childs > 0) $max_per_child += $phpfpm_child_max_mem_usage * $phpfpm_nb_childs / $apache_nb_childs;
		display("[..] Estimated avg memory usage per apache child : ".human_filesize($avg_per_child));
		display("[..] Estimated max memory usage per apache child : ".human_filesize($max_per_child));

		// potential memory usage with actual MaxRequestWorkers / MaxClients
		$potential_avg_mem_usage = ($actual_max_workers - $apache_nb_childs) * $avg_per_child;
		if ($potential_avg_mem_usage > $server_mem_available) {
			display("[!!] The actual average configuration of MaxRequestWorkers / MaxClients ($max_request_workers) may use more memory than available (".human_filesize($potential_avg_mem_usage)."/".human_filesize($server_mem_available).")");
		} else {
			display("[OK] There is enough avalaible memory to support the actual average configuration of MaxRequestWorkers / MaxClients ($max_request_workers) : (".human_filesize($potential_avg_mem_usage)."/".human_filesize($server_mem_available).")");
			// case of use biggest mem usage
			$potential_max_mem_usage = ($actual_max_workers - $apache_nb_childs) * $max_per_child;
			if ($potential_max_mem_usage > $server_mem_available) {
				display("[!!] The biggest usecase of MaxRequestWorkers / MaxClients ($max_request_workers) may use more memory than available (".human_filesize($potential_max_mem_usage)."/".human_filesize($server_mem_available).")");
			} else {
				display("[OK] There is enough avalaible memory to support the biggest usecase of MaxRequestWorkers / MaxClients ($max_request_workers) : (".human_filesize($potential_max_mem_usage)."/".human_filesize($server_mem_available).")");
			}
		}
	}

	// CPU usage
	$all_cpu = $apache_cpu + $phpfpm_cpu + $mysql_cpu;
	$cpu_per_child = $all_cpu / $apache_nb_childs;
	if (! $arg_c) {
		if ($all_cpu > $cpu_server_limit) {
			display("[!!] Actual CPU usage (".round($all_cpu, 2)."% / ".round($cpu_server_limit, 2)."%) is too high");
		}
		display("[..] Estimated avg CPU usage per Apache child : ".round($cpu_per_child, 2)."%");

		// potential CPU usage with actual MaxRequestWorkers
		$potential_cpu_usage = $actual_max_workers * $cpu_per_child;
		if ($potential_cpu_usage > $cpu_server_limit) {
			display("[!!] The actual configuration of MaxRequestWorkers / MaxClients ($max_request_workers) may use more CPU than the server can provide (".round($potential_cpu_usage, 2)."% / ".round($cpu_server_limit, 2)."%)");
		} else {
			display("[OK] The CPU is fast enough to support the actual configuration of MaxRequestWorkers / MaxClients ($max_request_workers) : (".round($potential_cpu_usage, 2)."% / ".round($cpu_server_limit, 2)."%)");
		}
	}

	// disk IO
	$all_io_rd = $apache_io_rd + $mysql_io_rd + $phpfpm_io_rd;
	$all_io_wd = $apache_io_wd + $mysql_io_wd + $phpfpm_io_wd;
	$all_io = $all_io_rd + $all_io_wd;
	$io_per_child = $all_io / $apache_nb_childs;
	if (! $arg_d) {
		display("[..] Estimated avg disk IO per Apache child : ".human_filesize($io_per_child)."/s");
		// potential IO usage with actual MaxRequestWorkers
		$potential_io_usage = ($actual_max_workers - $apache_nb_childs) * $io_per_child;
		if ($potential_io_usage > $disk_io_speed) {
			display("[!!] The actual configuration of MaxRequestWorkers / MaxClients ($max_request_workers) may use more IO than the disk can provide (".human_filesize($potential_io_usage)."/".human_filesize($disk_io_speed).")");
		} else {
			display("[OK] The disk is fast enough to support the actual configuration of MaxRequestWorkers / MaxClients ($max_request_workers) : (".human_filesize($potential_io_usage)."/".human_filesize($disk_io_speed).")");
		}
	}

	// **** Evaluate max possible value of MaxRequestWorkers / MaxClients  **** //

	$mem_avg_possible_workers = 999999999999999;
	$mem_max_possible_workers = 999999999999999;
	// compare MaxRequestWorkers / MaxClients to the max possible with this memory
	if (! $arg_m) {
		// max recommanded requestworkers : max 90% of available memory
		$mem_avg_possible_workers = $apache_nb_childs + ($server_mem_available * 90 / 100 / $avg_per_child);
		$mem_max_possible_workers = $apache_nb_childs + ($server_mem_available * 90 / 100 / $max_per_child);
		// floor to 10
		$mem_avg_possible_workers = floor($mem_avg_possible_workers / 10) * 10;
		$mem_max_possible_workers = floor($mem_max_possible_workers / 10) * 10;
	}

	// compare MaxRequestWorkers / MaxClients to the max possible with CPU
	if (! $arg_c) {
		$cpu_possible_workers = $cpu_server_limit / $cpu_per_child;
		// floor to 10
		$cpu_possible_workers = floor($cpu_possible_workers / 10) * 10;
		if ($cpu_possible_workers < $mem_avg_possible_workers) {
			$avg_possible_workers = $cpu_possible_workers;
		} else {
			$avg_possible_workers = $mem_avg_possible_workers;
		}
		if ($cpu_possible_workers < $mem_max_possible_workers) {
			$max_possible_workers = $cpu_possible_workers;
		} else {
			$max_possible_workers = $mem_max_possible_workers;
		}
	}

	// compare MaxRequestWorkers / MaxClients to the max possible with disk IO
	if (! $arg_d) {
		$io_possible_workers = 999999999999;
		if ($disk_io_speed != 0) {
			$io_possible_workers = $apache_nb_childs + ($disk_io_speed / $io_per_child);
			// floor to 10
			$io_possible_workers = floor($io_possible_workers / 10) * 10;
		}
		if ($io_possible_workers < $mem_avg_possible_workers) {
			$avg_possible_workers = $io_possible_workers;
		} else {
			$avg_possible_workers = $mem_avg_possible_workers;
		}
		if ($io_possible_workers < $mem_max_possible_workers) {
			$max_possible_workers = $io_possible_workers;
		} else {
			$max_possible_workers = $mem_max_possible_workers;
		}
	}

	if ($mod_mpm == "prefork") {
		if ($max_possible_workers == $avg_possible_workers) {
			display("[..] Maximum value for MaxRequestWorkers / MaxClients on this server is about $max_possible_workers");
		} else {
			display("[..] Maximum value for MaxRequestWorkers / MaxClients on this server is between $max_possible_workers and $avg_possible_workers");
		}
		if ($max_request_workers > $avg_possible_workers) {
			display("[>>] MaxClients / MaxRequestWorkers ($max_request_workers) should be smaller than $avg_possible_workers, you may adjust it accordingly.");
		} else {
			if ($max_request_workers >= $max_possible_workers) {
				display("[>>] MaxClients / MaxRequestWorkers ($max_request_workers) is bigger than $max_possible_workers, but smaller than $avg_possible_workers, this may work, but you will maybe have to lower it a little.");
			} else {
				display("[OK] MaxClients / MaxRequestWorkers ($max_request_workers) is smaller than $max_possible_workers");
			}
		}
	}

	if ($mod_mpm == "worker" || $mod_mpm == "event") {
		if ($max_possible_workers == $avg_possible_workers) {
			display("[..] Maximum value for MaxRequestWorkers / MaxClients on this server is about ".($max_possible_workers * $threads_per_child));
		} else {
			display("[..] Maximum value for MaxRequestWorkers / MaxClients on this server is between ".($max_possible_workers * $threads_per_child)." and ".($avg_possible_workers * $threads_per_child));
		}
		if ($actual_max_workers > $avg_possible_workers) {
			display("[>>] MaxClients / MaxRequestWorkers ($max_request_workers) should be smaller than ".($avg_possible_workers * $threads_per_child).", you may adjust it accordingly.");
		} else {
			if ($actual_max_workers >= $max_possible_workers) {
				display("[>>] MaxClients / MaxRequestWorkers ($max_request_workers) is bigger than ".($max_possible_workers * $threads_per_child).", but smaller than ".($avg_possible_workers * $threads_per_child).", this may work, but you will maybe have to lower it a little.");
			} else {
				display("[OK] MaxClients / MaxRequestWorkers ($max_request_workers) is smaller than ".($max_possible_workers * $threads_per_child));
			}
		}
	}
}

// ********************************************************
// *        Display all Apache config files content       *
// ********************************************************
if ($arg_a) {
	display_title("Apache config files content");
	echo "\r\n$conf_all\r\n)";
}
?>
